/**
 * @name:   Game of Life Common Header
 *
 * @author: Tomasz Gebarowski <gebarowski@gmail.com>
 * @date:   Wednesday 03 Dec 2008, 8:56 AM
 */

#ifndef __JCB_COMMON_H__
#define __JCB_COMMON_H__

#include <stdlib.h>
#include <stdio.h>
#include <config.h>
#include <sys/types.h>


/** -------------------------------------------------------------------------*/
/** --------------------------- Common Typedefs -----------------------------*/
/** -------------------------------------------------------------------------*/

/**
 * @brief Providing Boolean type for C
 */
typedef enum
{ 
    FALSE = 0,
    TRUE = 1
} bool;

/**
 * @brief Path for Gnuplot file
 */
#define GNUPLOT_FILE_PATH "plots"

/**
 * @brief Definitions of Boundry types
 */
typedef enum
{
    JCB_TOP_BOUNDARY = 0x1,
    JCB_BOTTOM_BOUNDARY = 0x2,
    JCB_OTHER_BOUNDARY = 0x4,
} jcb_boundry_t;

/**
 * @brief Error types definition
 */
typedef enum
{
    JCB_OK       = 0,  /* Everything is ok */
    JCB_EOF      = 1,  /* No more data to read. This is not an error */
    JCB_ENOMEM   = 2,  /* Not enough memory */
    JCB_EINVPTR  = 3,  /* Invalid Ptr, probably NULL ptr */
    JCB_EOUTB    = 4,  /* Index out of bound */
    JCB_ECMD     = 5,  /* Error while parsing command line argument */
    JCB_ERROR    = 6,  /* Generic Error */
    JCB_STATUS_MAX
} jcb_status_t;


/** -------------------------------------------------------------------------*/
/** --------------------------- API definitions -----------------------------*/
/** -------------------------------------------------------------------------*/



/**
 * @brief Macro returning ABS value of a given number
 */
#define ABS(x) ((x) >= 0 ? (x) : -(x))

/**
 * @brief Returns Maximum of two numbers
 */
#define MAX(x,y) ((x) >= (y) ? (x) : (y))

/**
 * @brief Macro definition for debug messages, to be easily removed when not needed
 */
#ifdef RELEASE_MODE
#define jcb_debug(s, ...)
#else
#define jcb_debug(s, ...) printf(s, ##__VA_ARGS__)
#endif

/**
 * @brief Macro definiton for printing normal messages
 */
#define jcb_message(s, ...) printf(s, ##__VA_ARGS__)


/**
 * @brief Get Neihbouring Rank Macros
 */
#define jcb_next_rank(rank,max)  (rank==max-1) ? 0   : (rank+1)
#define jcb_prev_rank(rank,max)  (rank==0)   ? max-1 : (rank-1)


/**
 * @brief Get Last array row pointer 
 */
#define jcb_last_row_ptr(arr,rows,col_size) (rows<=1) ? &arr[0] : (&arr[col_size * (rows - 1)])

/**
 * @brief Simple macro clearing screen
 */
#define jcb_clear()   printf("%c%s",27,"[2J"); printf("%c%s",27,"[1;1H");

/**
 * @brief Get string representation of status2008-11-20 Tomasz Gebarowski <tomasz.gebarowski@wirelesszt.com>

	* src/wzt-scmp-common/wzt_scmp_message_types.h
 *
 * @param  status Status
 *
 * @return Zero terminated string representing status
 */
const char *
jcb_status_to_str(jcb_status_t status);

/**
 * @brief Print status message
 *
 * @param status Status
 */
void
jcb_print_status(jcb_status_t status);


#endif /* __JCB_COMMON_H__ */



/** End of jcb_common.h */
