/**
 * @name:    Parallel Programming Timer Implementation
 *
 * @author: Tomasz Gebarowski <gebarowski@gmail.com>
 * @date:   Friday 21 Nov 2008, 5:36 PM
 */

#include <pp_timer.h>
#include <unistd.h>

/** -------------------------------------------------------------------------*/
/** ------------------------ Function Implementation ------------------------*/
/** -------------------------------------------------------------------------*/

/**
 * @brief Create pp_timer_t object and allocates memory for it
 */
pp_timer_t *
pp_timer_create()
{
    pp_timer_t * p_pp_timer = malloc(sizeof(pp_timer_t));

    if (p_pp_timer != NULL)
    {
        p_pp_timer->st_time = 0;
        p_pp_timer->en_time = 0;
    }

    return p_pp_timer;
}

/**
 * @brief Starts counting time
 */
void 
pp_timer_start(pp_timer_t *self)
{
    if (self != NULL)
    {
        self->st_time = clock();
    }
}

/**
 * @brief Stops counting time
 */
void 
pp_timer_stop(pp_timer_t *self)
{
    if (self != NULL)
    {
        self->en_time = clock();
    }
}


/**
 * @brief Resets timer
 *
 */
void 
pp_timer_reset(pp_timer_t *self)
{
    if (self != NULL)
    {
        self->en_time = 0;
        self->st_time = 0;
    }
}

/**
 * @brief Get measured time between start and stop
 *
 * @param Pointer to current instance of pp_timer_t object
 */
int
pp_timer_measured_time(pp_timer_t *self)
{
    if (self != NULL)
    {
        return (int)( 1000 * (self->en_time - self->st_time) / CLOCKS_PER_SEC );
    }

    return -1;
}

/**
 * @brief Disposes timer object
 */
void 
pp_timer_dispose(pp_timer_t *self)
{
    if (self != NULL)
    {
        free(self);
    }
}
