/**
 * @name:    Parallel Programming Timer Header
 *
 * @author: Tomasz Gebarowski <gebarowski@gmail.com>
 * @date:   Friday 21 Nov 2008, 5:36 PM
 */

#ifndef __PP_TIMER_H__
#define __PP_TIMER_H__

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/times.h>

/** -------------------------------------------------------------------------*/
/** --------------------------- Common Typedefs -----------------------------*/
/** -------------------------------------------------------------------------*/

typedef struct
{
    clock_t st_time;
    clock_t en_time;
} pp_timer_t;

/** -------------------------------------------------------------------------*/
/** --------------------------- API definitions -----------------------------*/
/** -------------------------------------------------------------------------*/

/**
 * @brief Create pp_timer_t object and allocates memory for it
 */
pp_timer_t *
pp_timer_create();

/**
 * @brief Starts counting time
 *
 * @param Pointer to current instance of pp_timer_t object
 */
void 
pp_timer_start(pp_timer_t *self);

/**
 * @brief Stops counting time
 *
 * @param Pointer to current instance of pp_timer_t object
 */
void 
pp_timer_stop(pp_timer_t *self);


/**
 * @brief Resets timer
 *
 * @param Pointer to current instance of pp_timer_t object
 */
void 
pp_timer_reset(pp_timer_t *self);


/**
 * @brief Get measured time between start and stop
 *
 * @param Pointer to current instance of pp_timer_t object
 *
 * @return Measured time in miliseconds, or -1 in case of error
 */
int
pp_timer_measured_time(pp_timer_t *self);


/**
 * @brief Disposes timer object
 *
 * @param Pointer to current instance of pp_timer_t object
 */
void 
pp_timer_dispose(pp_timer_t *self);


#endif /* __PP_TIMER_H__ */

/** End of pp_timer.h */
