/**
 * @name:   Game of Life Board Header
 *
 * @author: Tomasz Gebarowski <gebarowski@gmail.com>
 * @date:   Wednesday 3 Dec 2008, 8:59 AM
 */

#ifndef __JCB_BOARD_H__
#define __JCB_BOARD_H__

#include <stdlib.h>
#include <stdio.h>

/** -------------------------------------------------------------------------*/
/** --------------------------- Common Typedefs -----------------------------*/
/** -------------------------------------------------------------------------*/

/**
 * @brief Struct representing game board object
 */
typedef struct
{
    double        * p_arr;   
    double        * p_tmp_buf;
    unsigned int     rows;
    unsigned int     cols;

} jcb_board_t;


/** -------------------------------------------------------------------------*/
/** --------------------------- API definitions -----------------------------*/
/** -------------------------------------------------------------------------*/

/**
 * @brief Allocates memory for Game Board
 *
 * @param self Pointer to matrix instance
 *
 * @param rows
 * @param cols
 *
 * @return State of method invocation
 */
jcb_board_t *
jcb_board_create(unsigned int rows, 
                 unsigned int cols);

/**
 * @brief Allocates memory for tmp buffer
 *
 * @param self Pointer to matrix instance
 *
 * @param rows
 * @param cols
 *
 * @return State of method invocation
 */
jcb_status_t
jcb_board_buffer_create(jcb_board_t *self,
                        unsigned int rows, 
                        unsigned int cols);


/**
 * @brief Converts result from tmp buffer into square Matrix
 *        By removing rubbish which was created during MPI_Gather
 *
 * @param self Pointer to instance of Jcb Board
 * @param std_height  Standard height of row
 * @param max_height  Max height submatrix
 * @param last_height Height of last element in submatrix
 * @param parts     Number of Submatrixes which were before concatenation by Gather.
 *                  This is the number of processes working.
 *
 * @return State of method invocation
 */
jcb_status_t
jcb_board_convert(jcb_board_t *self,
                  unsigned int std_height,
                  unsigned int max_height,
                  unsigned int last_height,
                  unsigned int parts);


/**
 * @brief Print Game Board
 * 
 * @param self Pointer to game board instance
 *
 * @return State of method invocation
 */
void
jcb_board_print(jcb_board_t *self);

/**
 * @brief Writes final result to gnuplot data file
 * @param self Pointer to Game Board
 *
 * @return State of method invocation
 */
jcb_status_t
jcb_board_write_gnuplot(jcb_board_t *self);


/**
 * @brief Dispose Sub Board
 */
void
jcb_board_dispose(jcb_board_t *self);



#endif /* __JCB_BOARD_H__ */

/** End of jcb_board.h */
