/**
 * @name:   Game of Life Board Header
 *
 * @author: Tomasz Gebarowski <gebarowski@gmail.com>
 * @date:   Wednesday 3 Dec 2008, 8:59 AM
 */

#ifndef __JCB_SUB_BOARD_H__
#define __JCB_SUB_BOARD_H__

#include <stdlib.h>
#include <stdio.h>

/** -------------------------------------------------------------------------*/
/** --------------------------- Common Typedefs -----------------------------*/
/** -------------------------------------------------------------------------*/

/**
 * @brief Struct representing game sub_board object
 */
typedef struct
{
    double       * p_arr;   
    double       * p_tmp_src;      /** Temporary source board */
    double       * p_top_boundary; /** Stores Top boundary    */
    double       * p_bot_boundary; /** Stores Bottom boundary */
    unsigned char  boundary_type;  /** Flag indicating type of boundary */
    unsigned int   rows;
    unsigned int   cols;

} jcb_sub_board_t;


/** -------------------------------------------------------------------------*/
/** --------------------------- API definitions -----------------------------*/
/** -------------------------------------------------------------------------*/

/**
 * @brief Allocates memory for Game Board
 *
 * @param self Pointer to game sub_board instance
 *
 * @param rows
 * @param cols
 *
 * @return State of method invocation
 */
jcb_sub_board_t *
jcb_sub_board_create(unsigned int rows, 
                     unsigned int cols);


/**
 * @brief Initializes sub_board with 
 *        a proper boundary type
 *
 * @param self Pointer to current instance of Sub Board
 * @param boundary_type Boundry type
 *
 * @return State of method invocation
 */
jcb_status_t
jcb_sub_board_init(jcb_sub_board_t *self,
                   unsigned char boundary_type);



/**
 * @brief Calculates average of all values in this sub matrix
 *
 * @param self Pointer to current instance of Sub Board
 * @param avg  (OUT) Average value of sub matrix elements
 *
 * @return State of method invocation
 */
jcb_status_t
jcb_sub_board_avg(jcb_sub_board_t *self,
                  double          *value);


/**
 * @brief Creates Source Sub Board by concatenating
 *        first_row with sub_board and p_last_row
 */
jcb_status_t
jcb_sub_board_create_tmp_src(jcb_sub_board_t *self, 
                             double *p_first_row,
                             double *p_last_row,
                             unsigned int  row_size);

/**
 * @brief Runs a single iteration of game logic
 */
jcb_status_t
jcb_sub_board_run_step(jcb_sub_board_t *self);


/**
 * @brief Dispose Sub Board
 */
void
jcb_sub_board_dispose(jcb_sub_board_t *self);



#endif /* __JCB_SUB_BOARD_H__ */

/** End of jcb_sub_board.h */
